//modified by Jing Yang after 7/6/2001
#ifndef CSunburstCluster_H
#define CSunburstCluster_H

#include "RGB.H"
#include "colormap.H"
#include "CLinkList.H"

//COLORCOEF = (CMPSIZE-1) / 360. 
#define COLORCOEF 0.70833333;

class CSunburstCluster
{
public:
    //tree structure related fields;
    CSunburstCluster *parent;  //pointer to its parent;
    CSunburstCluster *children;  //pointer to its first children;
    CSunburstCluster *siblings;  //pointer to its next sibling;
    
    //display related fields;
    int entries;  //number of original dimensions contained in this node. leaf node, entries = 1; 
    int level;  //root node, level = 1; else level = parent->level+1;
    double start_angle;  //start angle in sunburst;
    double sweep_angle;  //sweep angle in sunburst;
    bool selected;
    bool show_children;
	bool current;  //current = true means this node need to be shown on screen;
    RGBt color;  //colormap[color_idx] is the color for this node in sunburst;

    double radius;  //cluster radius;
    
public:
	CSunburstCluster();
	~CSunburstCluster();
    
	//reset level of this node and its children and next silbings, find the maximum level among them;
	//current_level: level to be signed to this node; 
	//max_level: will be the maximum level of itself, its children and next silbings 
	void ResetLevels(int current_level, int &max_level);
    void Rotate(double degree);

    void ResetEntries();
	//reset the start_angle of this node;
	//start_angle: the new start angle to be asigned to this node; 
	//unit_angle: angle responds to one dimension;
	void ResetAngles(double start_angle, double unit_angle);

	//reset color of this node; color responds to its mid angle. 
	void ResetColors(colormap CMAP);

	void ResetCurrentList(CLinkList* current_list);

	//add a child to this node; the new child will be at the beginning of the child link list.
	//child: the new child to be added;
	void AddChild(CSunburstCluster *child);

	//add a silbing to this node; the new sibling will be its next sibling;
	//sibling: the new silbing to be added;
	void AddSibling(CSunburstCluster *sibling);

	//remove a child of this node;
	//rm_child: the child to be removed;
	void RemoveChild(CSunburstCluster *rm_child);

	//get the end angle of the node in sunburst;
	double GetEndAngle();

	//get the mid angle of the node in sunburst;
	double GetMidAngle();

    int GetEntries();

	double GetRadius();

	void ChangeSweepAngle(bool toward_left, double new_angle, double minimum_angle);

	//   |<-------------|
	//    delta_angle   start_angle
	void ChangeSweepAngleTowardLeft(double delta_angle, double minimum_angle);

	//   |------------->|
	//end_angle  delta_angle
	void ChangeSweepAngleTowardRight(double delta_angle, double minimum_angle);

	CSunburstCluster* FindNodeAccordingLevelAndAngle(int level, double angle);

	CSunburstCluster* FindNodeAccordingLevelAndAngle(int level, double angle, double eps, bool &start, bool &end, bool &in);

    void GetCenterPositon();

protected:

	//return the distance from start_angle to end_angle
	//  start_angle            end_angle
	//       | --  distance   ------|
	double GetAnglesDistance(double start_angle, double end_engle);

	void ChangeSweepAngle();

	//Increase sweep_angle by alpha, alpha > 0
	void IncreaseSweepAngle();

	//decrease sweep_angle by alpha, alpha < 0, sweep_angle should > entries*min_angle
	void DecreaseSweepAngle();

	//change start angle of this and all its children.
	void ChangeStartAngle(double new_start_angle);

	bool IsAngleIn(double angle);

	bool IsAngleIn(double angle, double eps, bool &start, bool &end, bool &in);

	bool IsAngleNear(double angle1, double angle, double eps);

	double StandarizeAngle(double angle);

	CSunburstCluster* FindLastSibling();

	CSunburstCluster* FindBeforeSibling();

};

#endif

